#include "xparameters.h"
#include "stdio.h"
#include "xio.h"
#include "xutil.h"
#include "fsl_math.h"
#include "xhwicap.h"
#include "xhwicap_parse.h"

#define XHI_EVEN_MAGIC_BYTE     0x0f
#define XHI_ODD_MAGIC_BYTE      0xf0
#define XHI_BIT_HEADER_FAILURE  -1
 
//====================================================
Xuint32 BitstreamLength;
static XHwIcap  HwIcap;	/* The instance of the HWICAP device */
	
Xuint32 ReadHeader(Xuint8 *Data) 
{
    Xuint32 I;   
    Xuint32 Len;  
    Xuint32 Tmp;  
 	 Xuint32 HeaderLength;
	 Xuint32 MagicLength;
	 Xuint8  *DesignName;
	 Xuint8  *StartAddress;
	 Xuint8  *PartName;
	 Xuint8  *Date;
	 Xuint8  *Time;

    /* Start Index at start of bitstream */
    BitstreamLength=0;
    StartAddress=Data;
    
    /* Initialize HeaderLength.  If header returned early inidicates
     * failure. 
     */
    HeaderLength = XHI_BIT_HEADER_FAILURE;

    /* Get "Magic" length */
    MagicLength = *Data++;
    MagicLength = (MagicLength << 8) | *Data++;
  
    /* Read in "magic" */
    for (I=0; I<MagicLength-1; I++) {
        Tmp = *Data++;
        if (I%2==0 && Tmp != XHI_EVEN_MAGIC_BYTE) 
        {
          return HeaderLength;   /* INVALID_FILE_HEADER_ERROR */
        }
        if (I%2==1 && Tmp != XHI_ODD_MAGIC_BYTE) 
        {
            return HeaderLength;   /* INVALID_FILE_HEADER_ERROR */
        }
    }

    /* Read null end of magic data. */
    Tmp = *Data++;
 
    /* Read 0x01 (short) */
    Tmp = *Data++;
    Tmp = (Tmp << 8) | *Data++;
 
    /* Check the "0x01" half word */
    if (Tmp != 0x01) 
    {
       return HeaderLength;   /* INVALID_FILE_HEADER_ERROR */
    }


    /* Read 'a' */
    Tmp = *Data++;
     if (Tmp != 'a') 
    {
        return HeaderLength;    /* INVALID_FILE_HEADER_ERROR  */
    }

    /* Get Design Name length */
    Len = *Data++;
    Len = (Len << 8) | *Data++;
 
    /* allocate space for design name and final null character. */
    DesignName = (char *)malloc(Len);

    /* Read in Design Name */
    for (I=0; I<Len; I++) 
    {
        DesignName[I] = *Data++;
    }

    /* Read 'b' */
    Tmp = *Data++;
    if (Tmp != 'b') 
    {
        return HeaderLength;  /* INVALID_FILE_HEADER_ERROR */
    }

    /* Get Part Name length */
    Len = *Data++;
    Len = (Len << 8) | *Data++;

    /* allocate space for part name and final null character. */
    PartName = (char *)malloc(Len);

    /* Read in part name */
    for (I=0; I<Len; I++) 
    {
        PartName[I] = *Data++;
    }

    /* Read 'c' */
    Tmp = *Data++;
    if (Tmp != 'c') 
    {
        return HeaderLength;  /* INVALID_FILE_HEADER_ERROR */
    }

    /* Get date length */
    Len = *Data++;
    Len = (Len << 8) | *Data++;

    /* allocate space for date and final null character. */
    Date = (char *)malloc(Len);

    /* Read in date name */
    for (I=0; I<Len; I++) 
    {
        Date[I] = *Data++;
    }

    /* Read 'd' */
    Tmp = *Data++;
     if (Tmp != 'd') 
    {
        return HeaderLength;  /* INVALID_FILE_HEADER_ERROR  */
    }

    /* Get time length */
    Len = *Data++;
    Len = (Len << 8) | *Data++;

    /* allocate space for time and final null character. */
    Time = (char *)malloc(Len);

    /* Read in time name */
    for (I=0; I<Len; I++) 
    {
        Time[I] = *Data++;
     }

    /* Read 'e' */
    Tmp = *Data++;
    if (Tmp != 'e') 
    {
        return HeaderLength;  /* INVALID_FILE_HEADER_ERROR */
    }

    /* Get byte length of bitstream */
    BitstreamLength = *Data++;
    BitstreamLength = (BitstreamLength << 8) | *Data++;
    BitstreamLength = (BitstreamLength << 8) | *Data++;
    BitstreamLength = (BitstreamLength << 8) | *Data++;
    
    HeaderLength = Data-StartAddress;
 
    return HeaderLength;
}
 
/* Loads a partial bitstream from FLASH into ICAP */
int XHwIcap_FLASH2Icap(Xuint32 StartAddress) 
{
	int i, numCharsRead, ace_buf_count, rc, SectorNumber;
//    XHwIcap_Bit_Header bit_header;
    Xuint8 data3,data2,data1,data0, * data_address;
    Xuint32 icap_buffer[2], HeaderLength;
    XStatus Status;
	 
#if DEBUG
	 print("In XHwIcap_FLASH2Icap\r\n");
	 xil_printf("StartAddress = %x\r\n",StartAddress);
#endif

    /* Read the bitstream header and determine bitstreamlength*/
    HeaderLength = ReadHeader((Xuint8 *) StartAddress);
    
	/* Now that we have info about the bitstream,skip the header, and start reading configuration data.
	 */
#if DEBUG
	 xil_printf("HeaderLength = %x, BitstreamLength = %x\r\n", HeaderLength,BitstreamLength);
#endif	     
	
    /* Loop through all bitstream data and write to ICAP */
    data_address = (Xuint8 *) (StartAddress + HeaderLength);
    for (i=0; i<BitstreamLength;i+=4) 
	 {

		  /* Convert 4 chars into an integer */
        data3 = *data_address++;
        data2 = *data_address++;
        data1 = *data_address++;
        data0 = *data_address++;
        icap_buffer[0] = ((data3 << 24) | (data2 << 16) | (data1 << 8) | (data0));
        i+=4;
		if(i<BitstreamLength) // store another word if even number of words
		{
			  /* Convert 4 chars into an integer */
	        data3 = *data_address++;
	        data2 = *data_address++;
	        data1 = *data_address++;
	        data0 = *data_address++;
	        icap_buffer[1] = ((data3 << 24) | (data2 << 16) | (data1 << 8) | (data0));
		}
		else
		{
			icap_buffer[1] = 0x0; // store 0- this is work around for hwicap bug in 11.4
		}

		Status = XHwIcap_DeviceWrite(&HwIcap, icap_buffer, 2);
        if (Status != XST_SUCCESS) 
        {
				/* Error writing to ICAP */
			  xil_printf("error writing to ICAP (%d)\r\n", Status);
           return -1;
        }
#if DEBUG
		xil_printf("In CF2ICAP ..Writing Word Number %d from current sector to ICAP..\r\n",i);
#endif
			
	}

#if DEBUG
	print("In CF2ICAP ..All Words Written to ICAP..\r\n");
#endif
}


int getNumber (){

	Xuint8 byte;
	Xuint8 uartBuffer[16];
	Xboolean validNumber;
	int digitIndex;
	int digit, number, sign;
	int c;	
	 
	while(1){
		byte = 0x00;
		digit = 0;
		digitIndex = 0;
		number = 0;
		validNumber = XTRUE;
	
		//get bytes from uart until RETURN is entered
	
		while(byte != 0x0d){
			byte = XUartLite_RecvByte(XPAR_RS232_UART_1_BASEADDR);
			uartBuffer[digitIndex] = byte; 
			XUartLite_SendByte(XPAR_RS232_UART_1_BASEADDR,byte);
			digitIndex++;
		}
			 
		//calculate number from string of digits
	
		for(c = 0; c < (digitIndex - 1); c++){
			if(c == 0){
				//check if first byte is a "-"
				if(uartBuffer[c] == 0x2D){
					sign = -1;
					digit = 0;
				}
				//check if first byte is a digit
				else if((uartBuffer[c] >> 4) == 0x03){
					sign = 1;
					digit = (uartBuffer[c] & 0x0F);
				}
				else	
					validNumber = XFALSE;	
			}
			else{
				//check byte is a digit
				if((uartBuffer[c] >> 4) == 0x03){  
					digit = (uartBuffer[c] & 0x0F);
				}
				else	
					validNumber = XFALSE;				
			}
			number = (number * 10) + digit;	
		}
		number *= sign;
		if(validNumber == XTRUE){
			return number;
		}
		print("This is not a valid number.\n\r");
	}
}

void get_operands(void)
{
	int first, second, result;
	
	print("First operand: ");
	first = getNumber();
	print("\r\n");
	print("Second operand: ");
	second = getNumber();
	print("\r\n");
	write_into_fsl(first, XPAR_FSL_FSL_MATH_0_INPUT_SLOT_ID);
	write_into_fsl(second, XPAR_FSL_FSL_MATH_0_INPUT_SLOT_ID);
	read_from_fsl(result, XPAR_FSL_FSL_MATH_0_OUTPUT_SLOT_ID);
	xil_printf("Result: %d\n\r",result);
	print("\r\n");
}

void menu(void)
{
	 xil_printf("--------------------------------------------------------\r\n");
	 xil_printf("      Press m or M for multiplication configuration\n\r");
	 xil_printf("      Press a or A for addition configuration\n\r");
	 xil_printf("      Press b or B for blanking configuration\n\r");
	 xil_printf("      Press o or O to enter operands and display result\n\r");
	 xil_printf("      Press q or Q to quit the demo\n\r");
}

int main (void) {

   char key;
   int Status, i;
   u32 ControlReg;
   XHwIcap_Config *ConfigPtr;
   u32 ConfigRegData;
   u32 num;
   
   print("-- Partial Reconfiguration Demo Using FSL Peripheral --\r\n");

	XIo_Out32(XPAR_FSL_RESET_BASEADDR,0x0000001);  // assert  fsl reset
	XIo_Out32(XPAR_FSL_RESET_BASEADDR,0x0000000);  // de-assert  fsl reset

	print("Before HwICAP Config\r\n");
	ConfigPtr = XHwIcap_LookupConfig(XPAR_XPS_HWICAP_0_DEVICE_ID);
	if (ConfigPtr == NULL) {
		return XST_FAILURE;
	}
	
	Status = XHwIcap_CfgInitialize(&HwIcap, ConfigPtr,
				ConfigPtr->BaseAddress);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}
	print("HWICAP Initialized\r\n");

    menu();
	key = XUartLite_RecvByte(XPAR_RS232_UART_1_BASEADDR);   		

    /* Keep performing user requested function until quit (6) is pressed. */
   while ((key != 'q') && (key != 'Q')) {
       switch (key) {
            case 'm': // Multiplier
            case 'M': // Multiplier
					xil_printf("\r\n      Performing reconfiguration for Multiplication\n\r");
					XHwIcap_FLASH2Icap(XPAR_FLASH_MEM0_BASEADDR+0x01f20000);
					XIo_Out32(XPAR_FSL_RESET_BASEADDR,0x0000001);  // assert  fsl reset
					XIo_Out32(XPAR_FSL_RESET_BASEADDR,0x0000002);  // de-assert  fsl reset and assert enable
					menu();
					break;
            case 'a': // Adder
            case 'A': // Adder
					xil_printf("\r\n      Performing reconfiguration for Addition\n\r");
					XHwIcap_FLASH2Icap(XPAR_FLASH_MEM0_BASEADDR+0x01f00000);
					XIo_Out32(XPAR_FSL_RESET_BASEADDR,0x00000001);  // assert reset
					XIo_Out32(XPAR_FSL_RESET_BASEADDR,0x00000002);  // de-assert reset and assert enable
					menu();
					break;
            case 'b': // Blanking
            case 'B': // Blanking
					xil_printf("\r\n      Performing reconfiguration for Blanking\n\r");
					XHwIcap_FLASH2Icap(XPAR_FLASH_MEM0_BASEADDR+0x01f40000);
					XIo_Out32(XPAR_FSL_RESET_BASEADDR,0x00000001);  // assert reset
					XIo_Out32(XPAR_FSL_RESET_BASEADDR,0x00000002);  // de-assert reset and assert enable
					menu();
					break;
  
            case 'o': // calculation
            case 'O': // calculation
					get_operands();
					menu();
					break;
           case 'q': // Quit
           case 'Q': // Quit
					break;
           default: // Quit or random data we ignore
					break;
       }
        /* Wait for another keystroke. */
       key = XUartLite_RecvByte(XPAR_RS232_UART_1_BASEADDR);
   }

   print("-- Exiting main() --\r\n");
   return 0;
}

