/*
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2007 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/


/***************************** Include Files *********************************/
#include <string.h>
 
#include <xparameters.h>
#include <xbasic_types.h>
#include <xstatus.h>
#include <stdio.h>
#include "xsysace_l.h"
#include "xsysace.h"
#include "sysace_stdio.h"
#include "xgpio.h"
#include "icap_interface.h"


#define XHI_EVEN_MAGIC_BYTE     0x0f
#define XHI_ODD_MAGIC_BYTE      0xf0
#define XHI_BIT_HEADER_FAILURE  -1
#define XHI_MAX_BUFFER_BYTES    4096
#define XHI_MAX_BUFFER_INTS     1024

XGpio gpio_icap_go_out, gpio_icap_done, gpio_icap_bitlength;
Xuint32 BitstreamLength;

Xuint32 ReadHeader(Xuint8 *Data, Xuint32 Size) 
{
    Xuint32 I;   
    Xuint32 Len;  
    Xuint32 Tmp;  
    Xuint32 Index;
	 Xuint32 HeaderLength;
	 Xuint32 MagicLength;
	 Xuint8  *DesignName;
	 Xuint8  *PartName;
	 Xuint8  *Date;
	 Xuint8  *Time;

    /* Start Index at start of bitstream */
    Index=0;

    /* Initialize HeaderLength.  If header returned early inidicates
     * failure. 
     */
    HeaderLength = XHI_BIT_HEADER_FAILURE;

    /* Get "Magic" length */
    MagicLength = Data[Index++];
    MagicLength = (MagicLength << 8) | Data[Index++];

    /* Read in "magic" */
    for (I=0; I<MagicLength-1; I++) {
        Tmp = Data[Index++];
        if (I%2==0 && Tmp != XHI_EVEN_MAGIC_BYTE) 
        {
          return HeaderLength;   /* INVALID_FILE_HEADER_ERROR */
        }
        if (I%2==1 && Tmp != XHI_ODD_MAGIC_BYTE) 
        {
            return HeaderLength;   /* INVALID_FILE_HEADER_ERROR */
        }
    }

    /* Read null end of magic data. */
    Tmp = Data[Index++];

    /* Read 0x01 (short) */
    Tmp = Data[Index++];
    Tmp = (Tmp << 8) | Data[Index++];

    /* Check the "0x01" half word */
    if (Tmp != 0x01) 
    {
       return HeaderLength;   /* INVALID_FILE_HEADER_ERROR */
    }


    /* Read 'a' */
    Tmp = Data[Index++];
    if (Tmp != 'a') 
    {
        return HeaderLength;    /* INVALID_FILE_HEADER_ERROR  */
    }

    /* Get Design Name length */
    Len = Data[Index++];
    Len = (Len << 8) | Data[Index++];

    /* allocate space for design name and final null character. */
    DesignName = (char *)malloc(Len);

    /* Read in Design Name */
    for (I=0; I<Len; I++) 
    {
        DesignName[I] = Data[Index++];
    }

    /* Read 'b' */
    Tmp = Data[Index++];
    if (Tmp != 'b') 
    {
        return HeaderLength;  /* INVALID_FILE_HEADER_ERROR */
    }

    /* Get Part Name length */
    Len = Data[Index++];
    Len = (Len << 8) | Data[Index++];

    /* allocate space for part name and final null character. */
    PartName = (char *)malloc(Len);

    /* Read in part name */
    for (I=0; I<Len; I++) 
    {
        PartName[I] = Data[Index++];
    }

    /* Read 'c' */
    Tmp = Data[Index++];
    if (Tmp != 'c') 
    {
        return HeaderLength;  /* INVALID_FILE_HEADER_ERROR */
    }

    /* Get date length */
    Len = Data[Index++];
    Len = (Len << 8) | Data[Index++];

    /* allocate space for date and final null character. */
    Date = (char *)malloc(Len);

    /* Read in date name */
    for (I=0; I<Len; I++) 
    {
        Date[I] = Data[Index++];
    }

    /* Read 'd' */
    Tmp = Data[Index++];
    if (Tmp != 'd') 
    {
        return HeaderLength;  /* INVALID_FILE_HEADER_ERROR  */
    }

    /* Get time length */
    Len = Data[Index++];
    Len = (Len << 8) | Data[Index++];

    /* allocate space for time and final null character. */
    Time = (char *)malloc(Len);

    /* Read in time name */
    for (I=0; I<Len; I++) 
    {
        Time[I] = Data[Index++];
    }

    /* Read 'e' */
    Tmp = Data[Index++];
    if (Tmp != 'e') 
    {
        return HeaderLength;  /* INVALID_FILE_HEADER_ERROR */
    }

    /* Get byte length of bitstream */
    BitstreamLength = Data[Index++];
    BitstreamLength = (BitstreamLength << 8) | Data[Index++];
    BitstreamLength = (BitstreamLength << 8) | Data[Index++];
    BitstreamLength = (BitstreamLength << 8) | Data[Index++];
	 
	 XGpio_DiscreteWrite(&gpio_icap_bitlength,1,BitstreamLength);  // write bistreamlength to output register

    HeaderLength = Index;

    return HeaderLength;

}

/** Loads a partial bitstream from CF into ICAP */
int CF2Icap(Xuint8* filename) 
{
	 int i, j, k, numCharsRead, ace_buf_count, icap_buf_count, rc;
	 SYSACE_FILE *stream;
     Xuint32 HeaderLength;
     Xuint8 data3,data2,data1,data0;
     Xuint8 systemACE_Buffer[XSA_CF_SECTOR_SIZE];
     Xuint32 word;
	 Xuint32 baseaddr;
     XStatus Status;

    /* Assign icap_buffer pointer */
	 baseaddr = (Xuint32 *)XPAR_ICAP_INTERFACE_0_BASEADDR;
	 /* Opening file */
    if ((stream = sysace_fopen(filename, "r")) == NULL) {
        xil_printf("Can't open file (%s)\r\n", filename);
        return -1;
    }
    
	 /* Read from systemAce */
	 numCharsRead = sysace_fread(systemACE_Buffer, 1, XSA_CF_SECTOR_SIZE, 
                                stream);
    if (numCharsRead <= 0) {
        xil_printf("Error reading from system ace (%d)\r\n", numCharsRead);
        return -1;
    }
  
	 /* Read the bitstream header */
    HeaderLength = ReadHeader(systemACE_Buffer,0);

	 /* close systemAce file handle */
	 rc = sysace_fclose (stream);
	 if (rc < 0) {
		/* Can't close */
		xil_printf("can't close file\r\n");
		return -1;
    }
//	 xil_printf("HeaderLength = %x, BitstreamLength = %x\r\n", HeaderLength,BitstreamLength);

	 /* Now that we have info about the bitstream,
	 * re-open and skip the header.
	 */
	     
    if ((stream = sysace_fopen(filename, "r")) == NULL) {
        /* Can't open file */
        xil_printf("can't open file\r\n");
        return -1;
    }

	 /* Read the header (effectively skipping it) */
	 numCharsRead = sysace_fread(systemACE_Buffer, 1, HeaderLength, 
                                stream);

	 /* Fill the buffer with the real bitstream info */
	 numCharsRead = sysace_fread(systemACE_Buffer, 1, XSA_CF_SECTOR_SIZE, 
                                stream);


    /* Fill first two words and write to ICAP */
    	k=1;
	 	ace_buf_count = 0;
    	icap_buf_count = 0;

    /* Fill 127 words into FIFO */
	   	for (i=0; i<508; i+=4) {
		  	data3 = systemACE_Buffer[ace_buf_count++];
		  	data2 = systemACE_Buffer[ace_buf_count++];
		  	data1 = systemACE_Buffer[ace_buf_count++];
		  	data0 = systemACE_Buffer[ace_buf_count++];
		  	word = ((data3 << 24) | (data2 << 16) | (data1 << 8) | (data0));
//			xil_printf("word %d= %0x\r\n",i/4,word);	
		  	/* Copy data to 32-bit reg */
		  	ICAP_INTERFACE_mWriteReg(baseaddr, 0, word);
	 	  	ICAP_INTERFACE_mWriteReg(baseaddr, 4, word);  // dummy write to create a we to icap processor
	   	}

	 	/* send icap_go pulse once */
	 	XGpio_DiscreteWrite(&gpio_icap_go_out,1,0xffffffff);  // send icap_go pulse
	 	XGpio_DiscreteWrite(&gpio_icap_go_out,1,0x0);  

    	for (i=508; i<BitstreamLength; i+=4) {
		  	/* Convert 4 chars into an integer */
	        data3 = systemACE_Buffer[ace_buf_count++];
	        data2 = systemACE_Buffer[ace_buf_count++];
	        data1 = systemACE_Buffer[ace_buf_count++];
	        data0 = systemACE_Buffer[ace_buf_count++];
	        word = ((data3 << 24) | (data2 << 16) | (data1 << 8) | (data0));
//			xil_printf("word %d= %0x\r\n",i/4,word);	

	        /* Copy data to 32-bit reg */
			ICAP_INTERFACE_mWriteReg(baseaddr, 0, word);
		  	ICAP_INTERFACE_mWriteReg(baseaddr, 4, word);  // dummy write to create a we to icap processor

		  	/* Check to see if we need to read from CF again */
	        if (ace_buf_count == XSA_CF_SECTOR_SIZE)
	        {
	            /* read next sector from CF */
	            numCharsRead = sysace_fread(systemACE_Buffer, 1, XSA_CF_SECTOR_SIZE,
	                                        stream);
	            ace_buf_count = 0;
	        }
	    }

		while(!(XGpio_DiscreteRead(&gpio_icap_done,1) & 0x00000001));  // wait for reconfig_done
	
		/* close systemAce file handle */
		rc = sysace_fclose (stream);
		if (rc < 0) {
			/* Can't close */
			xil_printf("can't close file\r\n");
			return -1;
	    }
  
	return 0;
}

void menu(void)
{
	 xil_printf("-- Xilinx Partial Reconfiguration Demo Using Custom ICAP Processor --\r\n");
	 xil_printf("      Press l or L for left shifting LED configuration\n\r");
	 xil_printf("      Press r or R for right shifting LED  configuration\n\r");
	 xil_printf("      Press b or B for blanking configuration\n\r");
	 xil_printf("      Press c or C for loading corrupted CRC word configuration\n\r");
	 xil_printf("      Press i or I for loading corrupted IDCODE word configuration\n\r");
	 xil_printf("      Press s or S for loading corrupted SYNC word configuration\n\r");
	 xil_printf("      Press q or Q to quit the demo\n\r");
}


int main()
{

    char key, key1;
    int rc, innerloop, Status;
	 XSysAce SysAce;
	 
    /* Print out title bar. */
	 xil_printf("\r\n\r\n");
	 xil_printf("Xilinx Partial Reconfiguration Demo Using Custom ICAP Processor \r\n");
	 XGpio_Initialize(&gpio_icap_go_out,XPAR_ICAP_GO_OUT_DEVICE_ID);
	 XGpio_Initialize(&gpio_icap_done,XPAR_ICAP_DONE_IN_DEVICE_ID);
	 XGpio_Initialize(&gpio_icap_bitlength,XPAR_ICAP_BITSTREAMLENGTH_DEVICE_ID);
	 XGpio_DiscreteWrite(&gpio_icap_go_out,1,0x0);
	 XGpio_DiscreteWrite(&gpio_icap_bitlength,1,0x0);
	 Status = XSysAce_Initialize(&SysAce, XPAR_SYSACE_COMPACTFLASH_DEVICE_ID);
	 if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	 }
	 print("SysAce Initialized \r\n");
	 
	 menu();

    /* Wait for information from UART (keypress). */
    key = XUartLite_RecvByte(XPAR_RS232_UART_1_BASEADDR);

    /* Keep performing user requested function until quit (6) is pressed. */
    while ((key != 'q') && (key != 'Q')) {
        switch (key) {
            case 'l': // Leftshift
            case 'L': // Leftshift
					xil_printf("\r\n      Performing reconfiguration for left shifting LEDs\n\r");
					CF2Icap("left.bit");
					menu();
					break;
            case 'r': // Rightshift
            case 'R': // Rightshift
					xil_printf("\r\n      Performing reconfiguration for right shifting LEDs\n\r");
					CF2Icap("right.bit");
					menu();
					break;
            case 'b': // Blanking
            case 'B': // Blanking
					xil_printf("\r\n      Performing reconfiguration for Blanking\n\r");
					CF2Icap("blank.bit");
					menu();
					break;
            case 'i': // IDECODE
            case 'I': // IDECODE
					xil_printf("\r\n      Performing reconfiguration with corrupted IDCODE \n\r");
					CF2Icap("idcode.bit");
					menu();
					break;
            case 'c': // CRC
            case 'C': // CRC
					xil_printf("\r\n      Performing reconfiguration with corrupted CRC \n\r");
					CF2Icap("crc.bit");
					menu();
					break;
            case 's': // Blanking
            case 'S': // Blanking
					xil_printf("\r\n      Performing reconfiguration with corrupted SYNC\n\r");
					CF2Icap("sync.bit");
					menu();
					break;
            case 'q': // Quit
            case 'Q': // Quit
					break;
            default: // Quit or random data we ignore
					break;
        }
        /* Wait for another keystroke. */
        key = XUartLite_RecvByte(XPAR_RS232_UART_1_BASEADDR);
    }

    /* User is done with us... */
    xil_printf("Thank you. \n\r");
}
