`timescale 1ns / 1ps
// Module Name:    icap_fsm 
//
//////////////////////////////////////////////////////////////////////////////////
module icap_fsm(clk, rst, pkt_go, data_avail, bitstream_done, rd_en, inc_cnt, RP_enable, RP_reset, ICAP_CE, ICAP_WE);
    input clk;
    input rst;
    input pkt_go;
    input data_avail;
    input bitstream_done;
	 output rd_en;
    output inc_cnt;
    output RP_enable;
    output RP_reset;
    output ICAP_CE;
    output ICAP_WE;

parameter [1:0] wait_for_start = 0, 
	process_pkt =1,
	process_pkt_1 = 2,
	reset_partition = 3; 
	
reg rd_en;
reg ICAP_WE;
reg ICAP_CE;
reg inc_cnt;
reg RP_enable;
reg RP_reset;

reg [1:0] state;

always @(posedge clk)
begin
	if(rst)
	begin
		state = wait_for_start;
		rd_en = 0;
		ICAP_WE = 1;
		ICAP_CE = 1;
		inc_cnt = 0;
		RP_enable = 0;
		RP_reset = 1;
	end
	else
	case (state)
	wait_for_start  : begin
							if(pkt_go)
							begin
								state = process_pkt;
							end
							else
							begin
								state = wait_for_start;
							end
							rd_en = 0;
							ICAP_WE = 1;
							ICAP_CE = 1;
							RP_enable = 1;
							RP_reset = 0;
							inc_cnt = 0;
				end
	process_pkt  : begin
							if(bitstream_done)
							begin
								state = reset_partition;
							end
							else if(data_avail)
							begin
								rd_en = 1;
								inc_cnt = 0;
								ICAP_WE = 1;
								ICAP_CE = 1;
								RP_enable = 0;
								RP_reset = 0;
								state = process_pkt_1;
							end
							else
							begin
								rd_en = 0;
								inc_cnt = 0;
								ICAP_WE = 1;
								ICAP_CE = 1;
								RP_enable = 0;
								RP_reset = 0;
								state = process_pkt;
							end
				end
	process_pkt_1  : begin
							if(bitstream_done)
							begin
								state = wait_for_start;
							end
							else if(data_avail)
							begin
								rd_en = 1;
								inc_cnt = 1;
								ICAP_WE = 0;
								ICAP_CE = 0;
								RP_enable = 0;
								RP_reset = 0;
								state = process_pkt_1;
							end
							else
							begin
								rd_en = 0;
								inc_cnt = 0;
								ICAP_WE = 1;
								ICAP_CE = 1;
								RP_enable = 0;
								RP_reset = 0;
								state = process_pkt;
							end
				end
	reset_partition: begin
								rd_en = 0;
								ICAP_WE = 1;
								ICAP_CE = 1;
								RP_enable = 1;
								RP_reset = 1;
								inc_cnt = 0;
								state = wait_for_start;
				end
	default: begin
					state = wait_for_start;
				end
   endcase
end				

endmodule
