`timescale 1ns / 1ps
// Module Name:    ICAP_top 
//
// RP_enable and RP_reset are for Reconfigurable Partition enable and reset respectively.  They are 
// positive logic assert
//////////////////////////////////////////////////////////////////////////////////
module icap_top(clk, rst, icap_go, empty, data, bitstreamlength, ICAP_CE, rd_en, reconfig_done, RP_enable, RP_reset);
    input clk;
    input rst;
    input icap_go;
	 input empty;
    input [31:0] data;
	 input [31:0] bitstreamlength;
	 output ICAP_CE; // for debugging
    output rd_en;
    output reconfig_done;
    output RP_enable;
    output RP_reset;


wire ICAP_CE;
wire rd_en;
reg reconfig_done;
wire reconfig_done_i;
wire [31:0] bitreverseddata;
(* KEEP="TRUE" *) wire busy;
(* KEEP="TRUE" *) wire [31:0] ICAP_dataout;

assign bitreverseddata = {data[24],data[25],data[26],data[27],data[28],data[29],data[30],data[31],
data[16],data[17],data[18],data[19],data[20],data[21],data[22],data[23],
data[8],data[9],data[10],data[11],data[12],data[13],data[14],data[15],
data[0],data[1],data[2],data[3],data[4],data[5],data[6],data[7]};
 
always @ (posedge clk)  
begin
	if(rst)
		reconfig_done <= 1'b0;
	else if(reconfig_done_i)
		reconfig_done <= 1'b1;
	else if(icap_go)
		reconfig_done <= 1'b0;
end
	 
icap_proc I1(
		.clk(clk), 
		.rst(rst), 
		.pkt_go(icap_go),
		.empty(empty),		
		.bitstreamlength(bitstreamlength), 
		.rd_en(rd_en),
		.ICAP_CE(ICAP_CE), 
		.bitstream_done(reconfig_done_i), 
		.RP_enable(RP_enable),
		.RP_reset(RP_reset)
		);

   ICAP_VIRTEX5 #(
      .ICAP_WIDTH("X32") // "X8", "X16" or "X32" 
   ) ICAP_VIRTEX5_inst (
      .BUSY(busy),      // Busy output
      .O(ICAP_dataout),            // 32-bit data output
      .CE(ICAP_CE),          // Clock enable input
      .CLK(clk),        // Clock input
      .I(bitreverseddata),            // 32-bit data input
      .WRITE(1'b0)     // Write input
   );
		
endmodule
