---------------------------------------------------------------
--
--  Partial Reconfiguration Reference Design
--
--  October 17, 2011: Parimal Patel, Xilinx Research Labs
--		For ML605 Evaluation board
--  				
--  This is the top-level for the PR example using ICAP.
-- 
--  Description:
--	 This top-level file instantiates the control module, a receive uart, a transmit
--       uart and one reconfig_module.  The reconfig_module can be loaded with either 
--       an adder flavor or a subtractor flavor.  There is also one global clock buffer.
--
---------------------------------------------------------------
 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity top is
    Port ( 	fpga_0_clk_1_sys_clk_p_pin  : in std_logic;		-- 200MHz system clock
				fpga_0_clk_1_sys_clk_n_pin  : in std_logic;		-- 200MHz system clock
				fpga_0_rst_1_sys_rst_pin : in std_logic;
				fpga_0_RS232_Uart_1_RX_pin : IN std_logic;
				fpga_0_SysACE_CompactFlash_SysACE_CLK_pin : IN std_logic;
				fpga_0_SysACE_CompactFlash_SysACE_MPIRQ_pin : IN std_logic;
				fpga_0_SysACE_CompactFlash_SysACE_MPD_pin : INOUT std_logic_vector(7 downto 0);
				fpga_0_RS232_Uart_1_TX_pin : OUT std_logic;
				fpga_0_SysACE_CompactFlash_SysACE_MPA_pin : OUT std_logic_vector(6 downto 0);
				fpga_0_SysACE_CompactFlash_SysACE_CEN_pin : OUT std_logic;
				fpga_0_SysACE_CompactFlash_SysACE_OEN_pin : OUT std_logic;
				fpga_0_SysACE_CompactFlash_SysACE_WEN_pin : OUT std_logic;
				button_start  : in std_logic;
				button_stop : in std_logic;
				led          : out std_logic_vector(3 downto 0)
         );	
end top;

architecture Behavioral of top is

-- reconfigurable module.  Controls LEDs
component rModule_leds is
   Port	(
      clk : in std_logic;
      reset : in std_logic;
      en : in std_logic;
      led : out std_logic_vector(3 downto 0)
   );
end component;

component led_control is
   Port	(
      clk : in std_logic;
      reset : in std_logic;
      button_start : in std_logic;
      button_stop : in std_logic;
      rst : out std_logic;
      en : out std_logic
   );
end component;

	COMPONENT system
	PORT(
		fpga_0_SysACE_CompactFlash_SysACE_CLK_pin : IN std_logic;
		fpga_0_SysACE_CompactFlash_SysACE_MPIRQ_pin : IN std_logic;
		fpga_0_RS232_Uart_1_RX_pin : IN std_logic;
		fpga_0_clk_1_sys_clk_p_pin : IN std_logic;
		fpga_0_clk_1_sys_clk_n_pin : IN std_logic;
		fpga_0_rst_1_sys_rst_pin : IN std_logic;    
		fpga_0_SysACE_CompactFlash_SysACE_MPD_pin : INOUT std_logic_vector(7 downto 0);      
		fpga_0_SysACE_CompactFlash_SysACE_MPA_pin : OUT std_logic_vector(6 downto 0);
		fpga_0_SysACE_CompactFlash_SysACE_CEN_pin : OUT std_logic;
		fpga_0_SysACE_CompactFlash_SysACE_OEN_pin : OUT std_logic;
		fpga_0_SysACE_CompactFlash_SysACE_WEN_pin : OUT std_logic;
		fpga_0_RS232_Uart_1_TX_pin : OUT std_logic;
		clock_generator_0_CLKOUT0_pin : OUT std_logic;
		icap_processor_0_RP_reset_pin : OUT std_logic;
		icap_processor_0_RP_enable_pin : OUT std_logic
		);
	END COMPONENT;

-- signals used in context logic
signal sys_clk : std_logic;

-- led signals
signal rst : std_logic;
signal en : std_logic;
signal icap_processor_0_RP_reset : std_logic;

begin

led_control_inst : led_control
   Port	map (
     clk => sys_clk,
     reset => icap_processor_0_RP_reset,
     button_start => button_start,
     button_stop => button_stop,
     rst => rst,
     en => en
);

reconfig_leds : rModule_leds
   Port	map (
     clk => sys_clk,
     reset => rst,
     en => en,
     led => led
);

system_i: system 
	Port map(
     fpga_0_RS232_Uart_1_RX_pin => fpga_0_RS232_Uart_1_RX_pin, 
     fpga_0_RS232_Uart_1_TX_pin => fpga_0_RS232_Uart_1_TX_pin, 
     fpga_0_SysACE_CompactFlash_SysACE_CLK_pin => fpga_0_SysACE_CompactFlash_SysACE_CLK_pin, 
     fpga_0_SysACE_CompactFlash_SysACE_MPA_pin => fpga_0_SysACE_CompactFlash_SysACE_MPA_pin, 
     fpga_0_SysACE_CompactFlash_SysACE_MPD_pin => fpga_0_SysACE_CompactFlash_SysACE_MPD_pin, 
     fpga_0_SysACE_CompactFlash_SysACE_CEN_pin => fpga_0_SysACE_CompactFlash_SysACE_CEN_pin, 
     fpga_0_SysACE_CompactFlash_SysACE_OEN_pin => fpga_0_SysACE_CompactFlash_SysACE_OEN_pin, 
     fpga_0_SysACE_CompactFlash_SysACE_WEN_pin => fpga_0_SysACE_CompactFlash_SysACE_WEN_pin, 
     fpga_0_SysACE_CompactFlash_SysACE_MPIRQ_pin => fpga_0_SysACE_CompactFlash_SysACE_MPIRQ_pin, 
     fpga_0_clk_1_sys_clk_p_pin => fpga_0_clk_1_sys_clk_p_pin, 
     fpga_0_clk_1_sys_clk_n_pin => fpga_0_clk_1_sys_clk_n_pin,
	  clock_generator_0_CLKOUT0_pin => sys_clk,
	  fpga_0_rst_1_sys_rst_pin => fpga_0_rst_1_sys_rst_pin, 
     icap_processor_0_RP_enable_pin => open,
     icap_processor_0_RP_reset_pin => icap_processor_0_RP_reset
);

end Behavioral;
