-- #####################################################
-- #
-- #  Partition-based Partial Reconfiguration Reference Design
-- #
-- #  Parimal Patel, Xilinx University Program, San Jose
-- #  October 6, 2011
-- #
-- #  Targeted for ML605 Evaluation Board
-- #
-- #####################################################


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Static Module to turn chip clock into a baud rate clock to be
--    used by uarts
entity uart_baudClock is
    Generic ( clock_rate : integer := 100000000;
               baud_rate : integer := 9600
            );

    Port (   in_clk : in std_logic;
           baud_clk : out std_logic
           );
end uart_baudClock;


architecture Behavioral of uart_baudClock is
begin

-- The following process controls the baud clock. The signal baud_clk is 
-- pulsed high for one clock cycle at a rate 16 times of the baud rate. 
-- Clock rate and baud rate are set in the top-level VHDL-file.
baudRate_process : process(in_clk)

-- Adjust the clock rate and the baud rate:
constant clock_rate : integer := 100000000;	-- 100 MHz
constant  baud_rate : integer := 9600;		-- 9600 buad	

-- keep track of clk_divide ticks of chip clock until we can generate 1 tick of 9600 baud
--    this should be less than 2048 (100000000 / 9600 * 16 = 100000000  651. 041....)
variable count : integer range 0 to 2047 := 0;
constant clk_divide : integer := clock_rate / (baud_rate * 16);

begin


--	Wait for a rising clock edge
	if in_clk'event and in_clk = '1' then
		count := count + 1;
		if count = clk_divide then
			count := 0;
			baud_clk <= '1';
		else
			baud_clk <= '0';
		end if;
	end if;
end process;

end Behavioral;
